@@ -1,3 +1,3 @@
-from .io import *
+from .io import io
 from .onedata import OneData
 from .plot import line_plot, bar_plot
