@@ -1,11 +1,14 @@
 import numpy as np
 import seaborn as sns
+import pandas as pd
 from pandas import Series
+from pandas import DataFrame
 import matplotlib.pyplot as plt
 from scipy.interpolate import interp1d
 
 
 class OneSeries(Series):
+
     def __init__(self, data):
         super().__init__(data=data)
 
@@ -14,6 +17,28 @@ class OneSeries(Series):
         from .onedata import OneData
         return OneData
 
+    def r_append(self, other):
+        """
+        Append another OneSeries data to the right of this series and return a new OneData.
+        :param other: the other OneSeries
+        :return: OneData
+        """
+        if isinstance(other, OneSeries):
+            return self._one_data({self.name: self, other.name: other})
+        elif isinstance(other, self._one_data) or isinstance(other, DataFrame):
+            return self._one_data(pd.concat([self, other], axis=1))
+
+    def l_append(self, other):
+        """
+        Append another OneSeries data to the left of this series and return a new OneData.
+        :param other: the other OneSeries
+        :return: OneData
+        """
+        if isinstance(other, OneSeries):
+            return self._one_data({other.name: other, self.name: self})
+        elif isinstance(other, self._one_data) or isinstance(other, DataFrame):
+            return self._one_data(pd.concat([other, self], axis=1))
+
     def summary(self, info: bool = True):
         """
         Return a summary of the whole OneSeries dataset.
@@ -39,9 +64,9 @@ class OneSeries(Series):
             sum_info['Memory Size(KB)'] = self.memory_usage() / 1024
 
             if info:
-                print('Unique: {}({:.2f}%)'.format(sum_info['Unique'], sum_info['Unique(%)']))
-                print('Missing: {}({:.2f}%)'.format(sum_info['Missing'], sum_info['Missing(%)']))
-                print('Zeros: {}({:.2f}%)'.format(sum_info['Zeros'], sum_info['Zeros(%)']))
+                print('Unique: {}({:.2f}%)'.format(sum_info['Unique'], sum_info['Unique(%)'] * 100))
+                print('Missing: {}({:.2f}%)'.format(sum_info['Missing'], sum_info['Missing(%)'] * 100))
+                print('Zeros: {}({:.2f}%)'.format(sum_info['Zeros'], sum_info['Zeros(%)'] * 100))
                 print('Means: {:.2f}'.format(sum_info['Means']))
                 print('Minimum: {}'.format(sum_info['Minimum']))
                 print('Maximum: {}'.format(sum_info['Maximum']))
@@ -58,20 +83,58 @@ class OneSeries(Series):
             sum_info['Memory Size(KB)'] = self.memory_usage() / 1024
 
             if info:
-                print('Unique: {}({:.2f}%)'.format(sum_info['Unique'], sum_info['Unique(%)']))
-                print('Missing: {}({:.2f}%)'.format(sum_info['Missing'], sum_info['Missing(%)']))
+                print('Unique: {}({:.2f}%)'.format(sum_info['Unique'], sum_info['Unique(%)'] * 100))
+                print('Missing: {}({:.2f}%)'.format(sum_info['Missing'], sum_info['Missing(%)'] * 100))
                 print('Entropy: {}'.format(sum_info['Entropy']))
                 print('Memory Size: {:.2f}KB'.format(sum_info['Memory Size(KB)']))
 
         return self._one_data(sum_info, index=[0])
 
+    def reverse(self, reset_index: bool = False):
+        """
+        Method for reversing the dataset
+        :param reset_index: true for reset the index of series
+        """
+        if reset_index:
+            return self.loc[::-1].reset_index(drop=True)
+        else:
+            return self.loc[::-1]
+
+    def shuffle(self, reset_index: bool = False, random_seed: int = None):
+        """
+        A non-inplace shuffle method.
+        :param reset_index: reset the index if it sets True
+        :param random_seed: the random seed of shuffle
+        :return: OneSeries
+        """
+        if reset_index:
+            return self.sample(frac=1, random_state=random_seed).reset_index(drop=True)
+        else:
+            return self.sample(frac=1, random_state=random_seed)
+
+    def top(self, n: int = 1):
+        """
+        Return the top of the values.
+        :param n: the number of value to return
+        """
+        return self.value_counts().nlargest(n=n)
+
+    def bottom(self, n: int = 1):
+        """
+        Return the bottom of the values
+        :param n: the number of value to return
+        """
+        return self.value_counts().nsmallest(n=n)
+
+    # ========================= Plot ========================= #
+
     def count_plot(self,
                    figsize: list = None,
                    title: str = None,
                    xlabel: str = None,
                    ylabel: str = None):
         """
-        Generate the count graph.
+        A fast-using generator of the count graph.
         """
         sns.set()
         plt.title(title)
@@ -91,7 +154,7 @@ class OneSeries(Series):
                   insert_num: int = 50,
                   show: bool = True):
         """
-        It's a fast plot function to generate graph rapidly.
+        A fast-using generator of the line graph.
         """
         sns.set()
         if figsize:
