@@ -19,17 +19,19 @@ class Plot:
         self = pd.DataFrame()
 
     def line_plot(self, x: str = None,
-                  y: list = None,
+                  y=None,
                   figsize: list = None,
                   title: str = None,
                   xlabel: str = None,
                   ylabel: str = None,
                   smooth: bool = False,
-                  insert_num: int = 50,
-                  label_loc: str = 'upper left',
+                  kind: str = 'cubic',
+                  interval: int = 50,
+                  legend_loc: str = 'upper left',
                   show: bool = True):
         """
         It's a fast plot function to generate graph rapidly.
+
         :param x: the x
         :param y: the y
         :param figsize: The size of figure.
@@ -37,43 +39,65 @@ class Plot:
         :param xlabel: label of x
         :param ylabel: label of y
         :param smooth: Set it True if the curve smoothing needed.
-        :param insert_num: define the number for smooth function.
-        :param label_loc: The location of the labels in plot.
+        :param kind: The method for smooth.
+        :param interval: define the number for smooth function.
+        :param legend_loc: The location of the legend in plot.
         :param show: plt.show will run if true
         """
         sns.set()
+
+        # data x pre-configuration process
+        # x will be used if x is specified, otherwise the default index would be used
+        if x:
+            x = self[x]
+        else:
+            x = list(self.index)
+
+        # data y pre-configuration process
+        # y can be a string or a list for multiple series plot
+        # oneline.Plot use inter1d for 1D data smoothing
+        if isinstance(y, str):
+            y_val = self[y]
+            if smooth:
+                x_new = np.linspace(min(x), max(x), len(x) * interval)
+                y_smooth = interp1d(x, y_val, kind=kind)
+                plt.plot(x_new, y_smooth(x_new), label=y)
+            else:
+                plt.plot(x, y_val, label=y)
+        elif isinstance(y, list):
+            for name in y:
+                y_val = self[name]
+                if smooth:
+                    x_new = np.linspace(min(x), max(x), len(x) * interval)
+                    y_smooth = interp1d(x, y_val, kind=kind)
+                    plt.plot(x_new, y_smooth(x_new), label=name)
+                else:
+                    plt.plot(x, y_val, label=name)
+
+        # the default plot if no argument input
+        else:
+            for name in list(self.columns):
+                y_val = self[name]
+                if smooth:
+                    x_new = np.linspace(min(x), max(x), len(x) * interval)
+                    y_smooth = interp1d(x, y_val, kind='cubic')
+                    plt.plot(x_new, y_smooth(x_new), label=name)
+                else:
+                    plt.plot(x, y_val, label=name)
+
+        # plt configuration
         if figsize:
             plt.figure(figsize=figsize)
         plt.title(title)
         plt.xlabel(xlabel)
         plt.ylabel(ylabel)
-        if x:
-            x = self[x]
-        else:
-            x = list(self.index)
-        if y:
-            for name in y:
-                y_val = self[name]
-                if smooth:
-                    x_new = np.linspace(min(x), max(x), len(x) * insert_num)
-                    y_smooth = interp1d(x, y_val, kind='cubic')
-                    plt.plot(x_new, y_smooth(x_new), label=name)
-                else:
-                    plt.plot(x, y_val, label=name)
-                plt.legend(loc=label_loc)
-        else:
-            for name in list(self.columns):
-                y_val = self[name]
-                if smooth:
-                    x_new = np.linspace(min(x), max(x), len(x) * insert_num)
-                    y_smooth = interp1d(x, y_val, kind='cubic')
-                    plt.plot(x_new, y_smooth(x_new), label=name)
-                else:
-                    plt.plot(x, y_val, label=name)
-                plt.legend(loc='upper left')
+        plt.legend(loc=legend_loc)
         if show:
             plt.show()
 
+        # return for advanced adjustment
+        return plt
+
     def count_plot(self, x: str = None,
                    hue: str = None,
                    figsize: list = None,
@@ -85,13 +109,8 @@ class Plot:
         :param figsize: The size of figure.
         :param show: plt.show will run if true
         """
-        try:
-            if not x:
-                self._raise_plot_value_error('x')
-        except Exception:
-            traceback.print_exc(limit=1, file=sys.stdout)
-            quit()
-
+        if not x:
+            self._raise_plot_value_error('x')
         sns.set()
         if figsize:
             plt.figure(figsize=figsize)
@@ -120,6 +139,8 @@ class Plot:
         if show:
             plt.show()
 
+        return plt
+
     def comparing_variables(self, variable1: str = None,
                             variable2: str = None,
                             show: bool = True):
@@ -139,6 +160,8 @@ class Plot:
         if show:
             plt.show()
 
+        return plt
+
 
 def line_plot(x: list = None,
               y: dict or list = None,
@@ -147,7 +170,8 @@ def line_plot(x: list = None,
               xlabel: str = None,
               ylabel: str = None,
               smooth: bool = False,
-              insert_num: int = 50,
+              kind: str = 'cubic',
+              interval: int = 50,
               show: bool = True):
     """
     That's a isolate line_plot for faster usage.
@@ -162,8 +186,8 @@ def line_plot(x: list = None,
     if isinstance(y, dict):
         for (name, i) in y.items():
             if smooth:
-                x_new = np.linspace(min(x), max(x), len(x) * insert_num)
-                y_smooth = interp1d(x, i, kind='cubic')
+                x_new = np.linspace(min(x), max(x), len(x) * interval)
+                y_smooth = interp1d(x, i, kind=kind)
                 if name:
                     plt.plot(x_new, y_smooth(x_new), label=name)
                 else:
@@ -175,7 +199,7 @@ def line_plot(x: list = None,
             plt.legend(loc='upper left')
     elif isinstance(y, list):
         if smooth:
-            x_new = np.linspace(min(x), max(x), len(x) * insert_num)
+            x_new = np.linspace(min(x), max(x), len(x) * interval)
             y_smooth = interp1d(x, y, kind='cubic')
             plt.plot(x_new, y_smooth(x_new))
         else:
@@ -183,6 +207,8 @@ def line_plot(x: list = None,
     if show:
         plt.show()
 
+    return plt
+
 
 def bar_plot(x: list = None,
              y: list = None,
@@ -204,3 +230,5 @@ def bar_plot(x: list = None,
     sns.barplot(x=x, y=y)
     if show:
         plt.show()
+
+    return plt
